"use client";

import { useState, useMemo } from "react";
import { ChevronDown, Download } from "lucide-react";
import { cn } from "@/lib/utils";
import {
  Collapsible,
  CollapsibleContent,
  CollapsibleTrigger,
} from "@/components/ui/collapsible";
import { Button } from "./ui/button";
import type { Result } from "@/lib/types";
import { handleExportCSV } from "@/utils/handleCSVExport";
import { RunItem } from "./run-item";

interface DateGroupProps {
  dateGroup: {
    date: string;
    runs: Result[];
  };
}

export function DateGroup({ dateGroup }: DateGroupProps) {
  const [isOpen, setIsOpen] = useState(true);

  // Calculate aggregate statistics for this date
  const stats = useMemo(() => {
    let totalSuccess = 0;
    let totalFailure = 0;

    dateGroup.runs.forEach((run) => {
      totalSuccess += run.success_count;
      totalFailure += run.failure_count;
    });

    const totalTests = totalSuccess + totalFailure;
    const successPercentage =
      totalTests > 0 ? (totalSuccess / totalTests) * 100 : 0;

    return {
      totalSuccess,
      totalFailure,
      totalTests,
      successPercentage,
    };
  }, [dateGroup]);

  return (
    <div className="mb-4 border border-zinc-600 rounded-lg overflow-hidden">
      <Collapsible
        defaultOpen={true}
        open={isOpen}
        onOpenChange={setIsOpen}
        className="w-full"
      >
        <CollapsibleTrigger className="flex w-full items-center justify-between py-4 text-left font-medium  hover:bg-zinc-600  focus-visible:ring-offset-2 px-4  transition-colors bg-zinc-700">
          <div className="flex items-center gap-4">
            <Button
              variant="default"
              size="sm"
              onClick={(e: React.MouseEvent) => {
                e.stopPropagation();
                handleExportCSV(dateGroup);
              }}
              className="flex items-center gap-2"
            >
              <Download className="h-4 w-4" />
            </Button>
            <span className="text-lg font-semibold">{dateGroup.date}</span>
          </div>
          <div className="flex items-center gap-4">
            <div className="flex items-center">
              <div className="rounded-full flex gap-1 pr-1"></div>
              <span className="text-xs text-zinc-400 ml-2">
                {stats.totalSuccess}/{stats.totalTests} passed
              </span>
            </div>
            <span className="text-sm text-zinc-200">
              {dateGroup.runs.length} runs
            </span>
            <ChevronDown
              className={cn(
                "h-4 w-4 text-zinc-100 transition-transform duration-200",
                isOpen && "rotate-180",
              )}
            />
          </div>
        </CollapsibleTrigger>
        <CollapsibleContent>
          <div className="divide-y divide-zinc-700">
            {dateGroup.runs
              .sort(
                (a, b) =>
                  new Date(b.date).getTime() - new Date(a.date).getTime(),
              )
              .map((run) => (
                <RunItem key={run.run_id} result={run} />
              ))}
          </div>
        </CollapsibleContent>
      </Collapsible>
    </div>
  );
}
