"use client";

import { useState } from "react";
import { ChevronDown } from "lucide-react";
import { cn } from "@/lib/utils";
import {
  Collapsible,
  CollapsibleContent,
  CollapsibleTrigger,
} from "@/components/ui/collapsible";
import type { Result } from "@/lib/types";

interface RunItemProps {
  result: Result;
}

export function RunItem({ result }: RunItemProps) {
  const [isOpen, setIsOpen] = useState(false);
  const hasChildren = result.children && result.children.length > 0;
  const totalTests = result.success_count + result.failure_count;
  const successPercentage =
    totalTests > 0 ? (result.success_count / totalTests) * 100 : 0;

  const formatSeconds = (milliseconds?: number): string => {
    if(!milliseconds) {
      return "--:--:--";
    }
    var seconds:number|string = milliseconds / 1000;
    var hours: number | string = Math.floor((seconds as number) / 3600);
    var minutes: number | string = Math.floor(
      ((seconds as number) - hours * 3600) / 60
    );
    seconds = (seconds as number) - hours * 3600 - minutes * 60;
    if (hours < 10) {
      hours = "0" + hours;
    }
    if (minutes < 10) {
      minutes = "0" + minutes;
    }
    if (seconds < 10) {
      seconds = "0" + seconds.toFixed(1);
    }
    var time = hours + ":" + minutes + ":" + seconds;
    return time;
  };

  return (
    <div className="w-full">
      <Collapsible
        defaultOpen={false}
        open={isOpen}
        onOpenChange={setIsOpen}
        className="w-full"
      >
        <CollapsibleTrigger className="flex w-full items-center justify-between py-3 text-left font-medium text-zinc-300 hover:bg-zinc-600 focus:outline-none focus-visible:ring-2 focus-visible:ring-gray-500 focus-visible:ring-offset-2 px-4  transition-colors">
          <div className="flex items-center gap-2">
            <span>{result.run_name}</span>
          </div>
          <div className="flex items-center gap-4">
            <div className="flex items-center">
              <div className="w-24 h-1.5 rounded-full overflow-hidden flex">
                <div
                  className="h-full bg-green-500"
                  style={{ width: `${successPercentage}%` }}
                ></div>
                <div
                  className="h-full bg-red-500"
                  style={{ width: `${100 - successPercentage}%` }}
                ></div>
              </div>
              <span className="text-xs text-zinc-400 ml-2">
                {result.success_count}/{totalTests} passed
              </span>
            </div>
            <span className="text-sm text-zinc-300">
              {formatSeconds(result.totalseconds)}
            </span>
            <ChevronDown
              className={cn(
                "h-4 w-4 text-gray-500 transition-transform duration-200",
                isOpen && "rotate-180"
              )}
            />
          </div>
        </CollapsibleTrigger>
        <CollapsibleContent>
          {hasChildren &&
            result.children.map((child, index) => (
              <div
                key={index}
                className="flex items-center justify-between py-3 px-4 pl-8 border-t border-zinc-700 hover:bg-zinc-700"
              >
                <div className="flex items-center gap-2">
                  <div
                    className={cn(
                      "w-2 h-2 rounded-full",
                      child.pending
                        ? "bg-blue-500"
                        : child.success
                        ? "bg-green-500"
                        : "bg-red-500"
                    )}
                  ></div>
                  <span className="text-zinc-200">{child.test_name}</span>
                  <span className="text-xs text-zinc-400">({child.path})</span>
                </div>
                <div className="flex items-center gap-4 pr-8">
                  <span className="text-sm text-zinc-300 w-12 text-right">
                    {formatSeconds(child.duration)}
                  </span>
                </div>
              </div>
            ))}
        </CollapsibleContent>
      </Collapsible>
    </div>
  );
}
