// This plugin makes API requests at specific points during Cypress test execution
// - Before any test in the entire test suite runs
// - Before each test file runs
// - After each test file completes

/**
 * Makes an API request with the provided configuration
 * @param {Object} options - Request options
 * @param {string} options.url - The URL to request
 * @param {string} options.method - HTTP method (GET, POST, etc.)
 * @param {Object} [options.body] - Request body for POST/PUT/PATCH requests
 * @param {Object} [options.headers] - Request headers
 * @param {string} eventType - Description of when this request is being made
 * @returns {Promise} - Promise resolving to the API response
 */
const makeApiRequest = async (options, eventType) => {
  console.log(`Making API request for event: ${eventType}`);

  try {
    const response = await fetch(options.url, {
      method: options.method || "GET",
      headers: {
        "Content-Type": "application/json",
        ...options.headers,
      },
      ...(options.body && { body: JSON.stringify(options.body) }),
    });

    const data = await response.json();
    console.log(
      `API request for ${eventType} completed with status: ${response.status}`,
    );
    console.log(data);
    return data;
  } catch (error) {
    console.error(`API request for ${eventType} failed:`, error);
    throw error;
  }
};

/**
 * Configuration for the different API requests to be made
 */
const apiRequests = {
  beforeAllTests: {},
  beforeTestFile: {},
  afterTestFile: {},
};

/**
 * Sets up the plugin in Cypress
 * @param {Object} on - Cypress event registration function
 * @param {Object} config - Cypress configuration
 */
// Track if event handlers have been registered to prevent duplicate registrations
let eventHandlersRegistered = false;

function setupAnalyticsPlugin(on, config) {
  // If event handlers are already registered, don't register them again
  if (eventHandlersRegistered) {
    console.log("API plugin event handlers already registered, skipping...");
    return;
  }
  // Track whether we've already run the before all tests request
  let beforeAllTestsExecuted = false;
  // Globals to store the current run ID and test ID
  let currentRunId, currentTestId, currentTestName = null;
  // Before all tests run (only once)
  on("before:run", async () => {
    if (!beforeAllTestsExecuted) {
      apiRequests.beforeAllTests = {
        url: `${config.historyTrackerURL || ""}/api/run/create`,
        method: "POST",
        body: { 
          // This Request creates a new Run, which can then be added
          // to with the beforeTestFile and afterTestFile requests
          runName: `Cypress Run @ ${new Date().toLocaleTimeString()}`,
        },
      };
      const res = await makeApiRequest(apiRequests.beforeAllTests,"beforeAllTests",);
      // Store the run ID for later use
      // This ID is used to associate all test files with this run
      // and is passed to the beforeTestFile and afterTestFile requests
      currentRunId = res.runId;
      beforeAllTestsExecuted = true;
    }
  });

  // Process individual test events
  on("before:spec", async (test) => {
    // If the run ID is not set, skip this request
    if (!currentRunId) return;
    // If the test name is the same as the last one, skip this request
    // This prevents duplicate requests for the same test this is 
    // important because the beforeTestFile request is made
    // twice when using cy.visit() (for some reason) (this is a bug in Cypress) 
    if (currentTestName === test.name) return;

    const testRequest = {
      url: `${config.historyTrackerURL || ""}/api/run/create`,
      method: "POST",
      body: {
        // This Request creates a new Test, it will be marked as pending
        // in the Database and on the Frontend
        runId: currentRunId,
        testName: test.name,
        path: test.relative,
      },
    };

    const res = await makeApiRequest(testRequest, "beforeTest");
    currentTestId = res.testId;
    currentTestName = test.name;
  });

  on("after:spec", async (test, results) => {
    if (!currentRunId) return;
    //  after:spec does not have the same bug as before:spec
    // so we don't need to check for duplicates
    const testRequest = {
      url: `${config.historyTrackerURL || ""}/api/run/create`,
      method: "POST",
      body: {
        runId: currentRunId,
        testId: currentTestId, // This request updates the Test created in beforeTestFile
        testName: test.name, 
        path: test.relative, 
        // This checks if there were any failures,
        // if there were, it marks the test as failed
        // otherwise it marks it as passed
        success: results.stats.failures === 0,
        // This is the duration of the test in milliseconds 
        duration: results.stats.duration,
      },
    };

    await makeApiRequest(testRequest, "afterTest");
  });

  // Mark event handlers as registered to prevent duplicate registrations
  eventHandlersRegistered = true;
  console.log("API plugin event handlers registered successfully");
}

module.exports = setupAnalyticsPlugin;
